/** @file
 * $Author: kusanagi $
 * $Date: 2006/02/07 17:11:09 $
 * $Revision: 6.0 $
 * 
 * Implementation of the Ac1InitReader class.
 */

#include "ReaderAlgorithms.h"

using namespace pamela::techmodel;

static log4cxx::LoggerPtr logger = log4cxx::Logger::getLogger(_T("pamela.techmodel.Ac1InitReader"));

/**
 * Constructor. 
 * @param  
 * @return 
 */
Ac1InitReader::Ac1InitReader(void): 
  TechmodelAlgorithm(PacketType::Ac1Init, "Ac1InitReader") { 
  logger->debug(_T("Constructor"));
  ac1Init = new Ac1InitEvent();
}

/**
 * Get a string with the version info of the algorithm.
 * @param
 * @return
 */
std::string Ac1InitReader::GetVersionInfo(void) const {
  return 
    "$Trailer: /home/cvsmanager/yoda/techmodel/Ac1InitReader.cpp,v 1.1.1.1 2004/07/06 12:20:23 Maurizio Nagni Exp $\n";
}


/**
 * Initialize the algorithm with a special run. This will initialize the
 * event reader routines for all packet types.
 * @param run 
 */
void Ac1InitReader::Init(PamelaRun *run) {
  logger->debug(_T("Initialize"));
  SetInputStream(run);
  run->WriteSubPacket(this, &ac1Init, ac1Init->Class());
}

/**
 * Unpack the Ac1Init event from an input file.
 * The CPU does not add any CRC control at the packet end.
 * @param EventNumber 
 * @param dataLength 
 */
void Ac1InitReader::RunEvent(int EventNumber, long int dataLength) throw (WrongCRCException){
    char      subData[dataLength];
    memset(subData,  0, dataLength*sizeof(char));
    InputFile->read(subData, sizeof(subData));
    ac1Init->ac1InitData        = new TArrayC(dataLength, subData);
}

